# Copyright (C) 2013-2023, Hiroshi Izumi (izumi.h@aist.go.jp)
# This code is part of the Biopython distribution and governed by its
# license.  Please see the LICENSE file that should have been included
# as part of this package.

from __future__ import print_function
#from Bio._py3k import basestring

import warnings

#from Bio.Alphabet import generic_protein
#from Bio.Data import SCOPData
from Bio.Seq import Seq
from Bio.PDB.PDBExceptions import PDBException
from Bio.PDB.vectors import calc_dihedral, calc_angle

import Bio.PDB
import os
from difflib import *

def conf_code(self):
    code_letter ={'A':'4c', 'B':'2a', 'C':'2c', 'D':'5a', 'E':'5c', \
             'F':'1a', 'G':'1c', 'H':'6a', 'I':'6c', 'J':'3a', 'K':'3c', \
             'L':'4a'}
    # ConfCode
    pi=3.141592653589793115997963468544
    pi1=pi/6
    pi2=2*pi/6
    pi3=3*pi/6
    pi4=4*pi/6
    pi5=5*pi/6
    mpi=-1*pi
    mpi1=-1*pi/6
    mpi2=-2*pi/6
    mpi3=-3*pi/6
    mpi4=-4*pi/6
    mpi5=-5*pi/6
    cc_arith = ""
    cc = ""
    # Decision of ConfCode
    if self>=0 and self<pi1:
        cc_arith = 'A'
        cc = "%s" % (code_letter[cc_arith])
    elif self>=pi1 and self<pi2:
        cc_arith = 'B'
        cc = "%s" % (code_letter[cc_arith])
    elif self>=pi2 and self<pi3:
        cc_arith = 'C'
        cc = "%s" % (code_letter[cc_arith])
    elif self>=pi3 and self<pi4:
        cc_arith = 'D'
        cc = "%s" % (code_letter[cc_arith])
    elif self>=pi4 and self<pi5:
        cc_arith = 'E'
        cc = "%s" % (code_letter[cc_arith])
    elif self>=pi5 and self<=pi:
        cc_arith = 'F'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi and self<=mpi5:
        cc_arith = 'G'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi5 and self<=mpi4:
        cc_arith = 'H'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi4 and self<=mpi3:
        cc_arith = 'I'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi3 and self<=mpi2:
        cc_arith = 'J'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi2 and self<=mpi1:
        cc_arith = 'K'
        cc = "%s" % (code_letter[cc_arith])
    elif self>mpi1 and self<0:
        cc_arith = 'L'
        cc = "%s" % (code_letter[cc_arith])
    else:
        cc_arith = None
        cc = None
    return cc
    
def get_phi_psi_omega_o_ssc_list(self):
    """Return the list of phi/psi/omega_o dihedral angles and supersecondary structure codes."""
    ppl=[]
    lng=len(self)
    for i in range(0, lng):
        res=self[i]
        try:
            n=res['N'].get_vector()
            ca=res['CA'].get_vector()
            c=res['C'].get_vector()
            o=res['O'].get_vector()
        except:
            # Some atoms are missing
            # Phi/Psi/Omega and SSC cannot be calculated for this residue
            ppl.append((None, None, None, None))
            res.xtra["PHI"]=None
            res.xtra["PSI"]=None
            res.xtra["OMEGA"]=None
            res.xtra["SSC"]=None
            continue
        # Phi
        if i>0:
            rp=self[i-1]
            try:
                cp=rp['C'].get_vector()
                phi=calc_dihedral(cp, n, ca, c)
            except:
                phi=None
        else:
            # No phi for residue 0!
            phi=None
        # Psi
        if i<(lng-1):
            rn=self[i+1]
            try:
                nn=rn['N'].get_vector()
                psi=calc_dihedral(n, ca, c, o)
            except:
                psi=None
        else:
            # No psi for last residue!
            psi=None
        # Omega
        if i<(lng-1):
            rn=self[i+1]
            try:
                nn=rn['N'].get_vector()
                nca=rn['CA'].get_vector()
                omega=calc_dihedral(o, c, nn, nca)
            except:
                omega=None
        else:
            # No omega for last residue!
            omega=None
        # SSC
        pi=3.141592653589793115997963468544
        pi1=pi/6
        pi2=2*pi/6
        pi3=3*pi/6
        pi4=4*pi/6
        pi5=5*pi/6
        mpi=-1*pi
        mpi1=-1*pi/6
        mpi2=-2*pi/6
        mpi3=-3*pi/6
        mpi4=-4*pi/6
        mpi5=-5*pi/6            
        try:
            # Decision of h for SSC
            if psi>=pi2 and psi<=pi:
                psic=1
                if phi<0 and phi>mpi5:
                    phic=1
                    if omega>=0 and omega<pi2:
                        omegac=1
                        ssc='h'
                    elif omega<0 and omega>mpi3:
                        omegac=1
                        ssc='h'
                    else:
                        omegac=0
                        ssc='o'
                else:
                    phic=0
                    ssc='o'
            elif psi<=mpi5 and psi>mpi:
                psic=1
                if phi<0 and phi>mpi5:
                    phic=1
                    if omega>=0 and omega<pi2:
                        omegac=1
                        ssc='h'
                    elif omega<0 and omega>mpi3:
                        omegac=1
                        ssc='h'
                    else:
                        omegac=0
                        ssc='o'
                else:
                    phic=0
                    ssc='o'
            # Decision of s for SSC
            elif psi>=0 and psi<pi2:
                psic=1
                if phi<=mpi1 and phi>mpi:
                    phic=1
                    if omega>=0 and omega<pi2:
                        omegac=1
                        ssc='s'
                    elif omega<0 and omega>mpi3:
                        omegac=1
                        ssc='s'
                    else:
                        omegac=0
                        ssc='o'
                else:
                    phic=0
                    ssc='o'
            elif psi<0 and psi>mpi3:
                psic=1
                if phi<=mpi1 and phi>mpi:
                    phic=1
                    if omega>=0 and omega<pi2:
                        omegac=1
                        ssc='s'
                    elif omega<0 and omega>mpi3:
                        omegac=1
                        ssc='s'
                    else:
                        omegac=0
                        ssc='o'
                else:
                    phic=0
                    ssc='o'
            else:
                psic=0
                ssc='o'
        except:
            ssc='T'                
        ppl.append((ssc, phi, psi, omega))
        # Add Phi/Psi/Omega and SSC to xtra dict of residue
        res.xtra["SSC"]=ssc
        res.xtra["PHI"]=phi
        res.xtra["PSI"]=psi
        res.xtra["OMEGA"]=omega
    return ppl

flist = os.listdir("C:/SSSCview/PDB_data")
one_letter ={'VAL':'V', 'ILE':'I', 'LEU':'L', 'GLU':'E', 'GLN':'Q', \
             'ASP':'D', 'ASN':'N', 'HIS':'H', 'TRP':'W', 'PHE':'F', 'TYR':'Y', \
             'ARG':'R', 'LYS':'K', 'SER':'S', 'THR':'T', 'MET':'M', 'ALA':'A', \
             'GLY':'G', 'PRO':'P', 'CYS':'C'}

for i, pdb_file in enumerate(flist):
    os.chdir("C:/SSSCview/PDB_data")
    for model in Bio.PDB.PDBParser().get_structure("PROTEIN", pdb_file) :
        for chain in model :
            polypeptides = Bio.PDB.PPBuilder().build_peptides(chain)
            for poly_index, poly in enumerate(polypeptides) :
                if poly_index == 0:
                    full_id=model.get_full_id()
                    mname = pdb_file[-8:-4]
                    fname = "C:/SSSCview/FASTA_ssch_data/%s_Model_%s_Chain_%s.txt" % (str(mname), str(model.id), str(chain.id))
                    fc = open(str(fname), 'w')
                    fc.write('')
                    mname1 = ">%s_Model_%s_Chain_%s " % (str(mname), str(model.id), str(chain.id))
                    mname2 = "(part %i of %i) " % (poly_index+1, len(polypeptides))
                    mname3 = "length %i " % (len(poly))
                    mname4 = "from %s%i " % (poly[0].resname, poly[0].id[1])
                    mname5 = "to %s%i" % (poly[-1].resname, poly[-1].id[1])
                    fc.write(str(mname1))
                    fc.write('\n')
                    for res_index, residue in enumerate(poly) :
                        res_name1 = "%s" % (residue.resname)
                        res_name2 = "%s" % (one_letter[res_name1])
                        fc.write('')
                        fc.write(res_name2)
                    fc.write('X')
                    if poly_index == len(polypeptides) - 1:    
                        fc.write('\n')
                    else:
                        continue
                    fc.close()
                else:
                    full_id=model.get_full_id()
                    mname = pdb_file[-8:-4]
                    fname = "C:/SSSCview/FASTA_ssch_data/%s_Model_%s_Chain_%s.txt" % (str(mname), str(model.id), str(chain.id))
                    fc = open(str(fname), 'a')
                    fc.write('')
                    mname1 = ">%s_Model_%s_Chain_%s " % (str(mname), str(model.id), str(chain.id))
                    mname2 = "(part %i of %i) " % (poly_index+1, len(polypeptides))
                    mname3 = "length %i " % (len(poly))
                    mname4 = "from %s%i " % (poly[0].resname, poly[0].id[1])
                    mname5 = "to %s%i" % (poly[-1].resname, poly[-1].id[1])
                    for res_index, residue in enumerate(poly) :
                        res_name1 = "%s" % (residue.resname)
                        res_name2 = "%s" % (one_letter[res_name1])
                        fc.write('')
                        fc.write(res_name2)
                    fc.write('X')
                    if poly_index == len(polypeptides) - 1:    
                        fc.write('\n')
                    else:
                        continue
                    fc.close()
            for poly_index, poly in enumerate(polypeptides) :
                full_id=model.get_full_id()
                mname = pdb_file[-8:-4]
                fname = "C:/SSSCview/FASTA_ssch_data/%s_Model_%s_Chain_%s.txt" % (str(mname), str(model.id), str(chain.id))
                fc = open(str(fname), 'a')
                fc.write('')
                mname1 = ">%s_Model_%s_Chain_%s " % (str(mname), str(model.id), str(chain.id))
                mname2 = "(part %i of %i) " % (poly_index+1, len(polypeptides))
                mname3 = "length %i " % (len(poly))
                mname4 = "from %s%i " % (poly[0].resname, poly[0].id[1])
                mname5 = "to %s%i" % (poly[-1].resname, poly[-1].id[1])
                #phi_psi_omega_o_ssc = poly.get_phi_psi_omega_o_ssc_list()
                phi_psi_omega_o_ssc = get_phi_psi_omega_o_ssc_list(poly)
                for res_index, residue in enumerate(poly) :
                    res_name = "%s%i" % (residue.resname, residue.id[1])
                    l_phi_psi_omega_o_ssc = phi_psi_omega_o_ssc[res_index]
                    ssc = l_phi_psi_omega_o_ssc[0]
                    if ssc:
                        sscH = ssc.replace('h', 'H')
                        sscHS = sscH.replace('s', 'S')
                        sscHST = sscHS.replace('o', 'T')
                        fc.write('')
                        fc.write(sscHST)
                    else:
                        fc.write('D')
                fc.write('D')
                fc.close()

for i, pdb_file in enumerate(flist):
    os.chdir("C:/\SSSCview/PDB_data")
    for model in Bio.PDB.PDBParser().get_structure("PROTEIN", pdb_file) :
        for chain in model :
            polypeptides = Bio.PDB.PPBuilder().build_peptides(chain)
            if polypeptides:
                os.chdir("C:/SSSCview")
                full_id=model.get_full_id()
                mname = pdb_file[-8:-4]
                fname = "C:/SSSCview/ConfCode_mchain/%s_Model_%s_Chain_%s_mchain.txt" % (str(mname), str(model.id), str(chain.id))
                fc = open(str(fname), 'w')
                fc.write('')
                fc.close()
                for poly_index, poly in enumerate(polypeptides) :
                    #os.chdir("C:\\SSSC_Analysis")
                    #full_id=model.get_full_id()
                    #mname = pdb_file[-8:-4]
                    #fname = "C:\\SSSC_Analysis\ConfCode_mchain\%s_Model_%s_Chain_%s_mchain.txt" % (str(mname), str(model.id), str(chain.id))
                    fc = open(str(fname), 'a')
                    fc.write('')
                    mname1 = "%s_Model_%s_Chain_%s " % (str(mname), str(model.id), str(chain.id))
                    mname2 = "(part %i of %i) " % (poly_index+1, len(polypeptides))
                    mname3 = "length %i " % (len(poly))
                    mname4 = "from %s%i " % (poly[0].resname, poly[0].id[1])
                    mname5 = "to %s%i" % (poly[-1].resname, poly[-1].id[1])
                    fc.write(str(mname1))
                    fc.write(str(mname2))
                    fc.write(str(mname3))
                    fc.write(str(mname4))
                    fc.write(str(mname5))
                    fc.write('\n')
                    
                    #phi_psi_omega_o_ssc_list = poly.get_phi_psi_omega_o_ssc_list()
                    phi_psi_omega_o_ssc_list = get_phi_psi_omega_o_ssc_list(poly)
                    for res_index, residue in enumerate(poly) :
                        res_name = "%s%i" % (residue.resname, residue.id[1])
                        l_phi_psi_omega_o_ssc_list = phi_psi_omega_o_ssc_list[res_index]
                        #print l_phi_psi_omega_o_ssc_list 
                        ssc = l_phi_psi_omega_o_ssc_list[0]
                        phi = l_phi_psi_omega_o_ssc_list[1]
                        psi = l_phi_psi_omega_o_ssc_list[2]
                        omega = l_phi_psi_omega_o_ssc_list[3]
                        try:
                            ccphi = conf_code(phi)
                        except:
                            ccphi = None
                        try:
                            ccpsi = conf_code(psi)
                        except:
                            ccpsi = None
                        try:
                            ccomega = conf_code(omega)
                        except:
                            ccomega = None
                        if ssc:
                            sscH = ssc.replace('h', 'H')
                            sscHS = sscH.replace('s', 'S')
                            sscHST = sscHS.replace('o', 'T')
                            #fc.write('')
                            #fc.write(sscHST)
                        else:
                            #fc.write('D')
                            sscHST = 'D'
                        fc.write(str(res_name))
                        fc.write(', ')
                        fc.write(str(sscHST))
                        fc.write(', ')
                        fc.write(str(ccphi))
                        fc.write(', ')
                        fc.write(str(ccpsi))
                        fc.write(', ')
                        fc.write(str(ccomega))
                        fc.write('\n')
                    fc.write('\n')
                    fc.close()

